/*
 * ----------------------------------------------------------------------------
 *
 * Copyright (C) March 1996 Jochen Karrer
 * 
 * Released under the terms of the GPL.
 *
 * IO-functions for the MV1000 devicedriver. They are used in the loadable
 * Kernel module. But there should be no problem to use them in User space
 * also. Only  pointers to the memory mapped IO of the MV-1000's  are needed
 * 
 * ----------------------------------------------------------------------------
 */

/*
 * --------------------------------------------------------------------------------
 *
 * write/read from/to registers
 *
 * -------------------------------------------------------------------------------
 */
static  inline  void
mv1_outl(u32 value,u_long reg,MV1000 *mv1) {
	if(mv1->simulation)
		return;
	if(reg < 1024*1024*4)
		writel(value,mv1->iomem + reg);
	readl(mv1->iomem);
	udelay(1);
}

static   inline u32
mv1_inl(u_long reg,MV1000 *mv1) {
	u32 ret=0;
	if(reg < 1024*1024*4)
		ret = readl(mv1->iomem + reg );
	return ret;
}
/*
 * -----------------------------------------------------------------------------
 *
 * Registers of the MV1000  are 32 Bit. Because Windows is still existing
 * there is a 16 Bit Driver for the MV-1000. Because registers  would be
 * in an indeterminate state if the access is split into two 16 bit writes,
 * The data is latched in the EXTERN_DATA register and then strobed into
 * the wanted register.
 *
 * ----------------------------------------------------------------------------- 
 */
static void 
mv1_extoutl(u32 value,u_long reg,MV1000 *mv1) {
        u_long flags;
        save_flags(flags);
        cli();
        mv1_outl(value,EXTERN_DATA,mv1);
        mv1_outl(0,reg,mv1);
        restore_flags(flags);
}
/*
 * ---------------------------------------------------------------------------
 *
 * The MV1000  has a bus for some external Chips. The upper 16 Bits of
 * the PB_BUS register are decoded to select the chip, the lower 16
 * contain the data. For serial chips Bit 0 is SERIAL_DATA and Bit 1 is
 * serial clock. 
 *
 * ---------------------------------------------------------------------------
 */
static inline void
mv1_pboutw(u16 value,u16 addr,MV1000 *mv1){
        mv1_extoutl( (u32)value + ((u32)addr<<16),PB_BUS,mv1); 
}
static  inline void
mv1_lutctl_out(u16 value,MV1000 *mv1){
	mv1_pboutw(value,LUT_CTL_ADR,mv1);
}
/*
 * ---------------------------------------------------------------------
 *
 *  lm1882_out --
 *
 *	write to a register of lm1882. Use the manual addressing mode
 *
 * Cycle		Load Faling Edge		Load Rising Edge
 * ---------------------------------------------------------------------
 *   1			Enable Manual Addressing	Load Address 
 *   2			EnableLbyte Data Load		Load Lbyte 
 *   3			EnableHbyte Data Load		Load Hbyte 
 *   		 
 *	
 * ---------------------------------------------------------------------
 */
static void // Untested I hope this works. I don't know polarity of LOAD
lm1882_out(u16 data,u8 adr,MV1000 *mv1){
	u_long flags;
	u8 lbyte = data & 0xff;
	u8 hbyte = (data >> 8) & 0xf;
	static char warn = 1;
	if(adr > 19) {
		if(warn) {
			printk("warning lm1882 called with adr %d\n",adr); 
			warn--; 
		}
		return;
	}
	save_flags(flags);
	cli();
	mv1_pboutw(LM1882_ADDR,0,mv1);
	mv1_pboutw(LM1882_ADDR,LM1882_LOAD,mv1);
	mv1_pboutw(adr | LM1882_ADDR,LM1882_LOAD,mv1);
	mv1_pboutw(adr | LM1882_ADDR,0,mv1);
	
	mv1_pboutw(LM1882_L_BYTE,0,mv1);
	mv1_pboutw(LM1882_L_BYTE,LM1882_LOAD,mv1);
	mv1_pboutw(lbyte | LM1882_L_BYTE,LM1882_LOAD,mv1);
	mv1_pboutw(lbyte | LM1882_L_BYTE,0,mv1);

	mv1_pboutw(LM1882_H_BYTE,0,mv1); 
	mv1_pboutw(LM1882_H_BYTE,LM1882_LOAD,mv1);
	mv1_pboutw(hbyte | LM1882_H_BYTE,LM1882_LOAD,mv1);
	mv1_pboutw(hbyte | LM1882_H_BYTE,0,mv1);
	mv1->lm1882[adr]=data; 
	restore_flags(flags);
}
/*
 * -------------------------------------------------------------------------
 *
 * mv1_dac_out --
 *
 *	writing to the registers of the DAC-8800 D/A converter	
 *
 * -------------------------------------------------------------------------
 */
static  void
mv1_dac_out(u8 data, u8 dac,MV1000 *mv1){
        u_long flags;
        u16 value = data + ((dac & 7)<<8);
        int i;
        save_flags(flags);
        cli();
	mv1_pboutw(0,0,mv1);
        for(i=0;i<12;i++){
                mv1_pboutw( ((value>>11) & 1),0,mv1);
                mv1_pboutw( ((value>>11) & 1)|SERIAL_CLOCK,0,mv1);
                value <<= 1;
        }
        mv1_pboutw(0,DAC_CS,mv1);
	mv1_pboutw(0,0,mv1);
	mv1->dac[dac&7] = data;
        restore_flags(flags);
}
/*
 * ---------------------------------------------------------
 *
 * vctl_fifo_out 
 *
 *	Write to the 40 Bit strobed fifo. 
 *      The end of the 16 Bit fifo 
 * 	in the MV-1000 is connected to a 24 bit fifo on 
 *      the MV-1100
 *      THIS FUNCTION IS NOT TESTED WITH THE MV1100 
 *
 * --------------------------------------------------------- 
 */
static  void
vctl_fifo_out(u32 ext_vctl,u16 vctl,MV1000 *mv1){
        int i;
        u_long flags;
        save_flags(flags);
        cli();
	mv1->ext_vctl = ext_vctl;
        mv1_pboutw(0, 0,mv1);
        for(i=0;i<24;i++){
                mv1_pboutw( ((ext_vctl>>23) & 1),0,mv1);
                mv1_pboutw( ((ext_vctl>>23) & 1)|SERIAL_CLOCK,0,mv1);
                ext_vctl <<= 1;
        }
	mv1->vctl = vctl;
        for(i=0;i<16;i++){
                mv1_pboutw( ((vctl>>15) & 1),0,mv1);
                mv1_pboutw( ((vctl>>15) & 1)|SERIAL_CLOCK,0,mv1);
                vctl <<= 1;
        }
        mv1_pboutw(0,VCTL_REG,mv1); // Strobe in
        mv1_pboutw(0,0,mv1);
        restore_flags(flags);
}

static  inline void
mv1_vctl_outw(u16 value,MV1000 *mv1){
        u_long flags;
        save_flags(flags);
	cli();
	vctl_fifo_out(mv1->ext_vctl,value,mv1);	
        restore_flags(flags);
}

static  inline void
ext_vctl_out(u32 value,MV1000 *mv1){
        u_long flags;
        save_flags(flags);
	cli();
	vctl_fifo_out(value,mv1->vctl,mv1);	
        restore_flags(flags);
}

#define mv11_out(val,mv1) ext_vctl_out(val,mv1)

/*
 * ---------------------------------------------------------------------------
 * BT254 has 3 Bit address and 8 Bit Databus
 *
 * Bit 0..7 are the data
 * Bit 8 is *RD/WR
 * Bit 11,12,13 are the adr
 * Bit 14 strobes the output of bt254 into the fifo (when high ?)
 * Bit 15 strobes the contents of the fifo to the parallel output (when low ?)?
 *
 * This function doesn't work actually.
 *
 * -----------------------------------------------------------------------------
 */

static void
bt254_out(u32 val,u32 adr,MV1000 *bs) {
        u32 output = bs->ext_vctl & ~(0xff|7<<11);
        output = output | (val&0xff) | ((adr&7)<<11);
        output = output | (1<<8);               // set to write
        output |= 0x8000;                       // strobe =1;
        output &= ~(1<<14);                      // do not read to fifo

        ext_vctl_out(output,bs);
        output &= ~(1<<15);                      // strobe to parallel output
        ext_vctl_out(output,bs);
        output |= (1<<15);                       // strobe = 1 (done)
        ext_vctl_out(output,bs);

}


/*
 * ----------------------------------------------------------------
 *
 * mv1_config_inw --
 * 
 * Read the configuration of the card ( Jumpers, Modules ) 
 * from serial port. Can be read after a full write of VCTL 
 *
 * ----------------------------------------------------------------
 */

static u16
mv1_config_inw(MV1000 *mv1){
	int i;
	u16 config=0;
	u_long flags;
        save_flags(flags);
        cli();
	mv1_vctl_outw(mv1->vctl,mv1);
	for(i=0;i<16;i++){
		config <<= 1;
		if(mv1_inl(CURR_LINE,mv1)&SERIAL_DATA_IN_MASK) {
			config |= 1;
		}	
		mv1_pboutw( 0,0,mv1);
		mv1_pboutw( SERIAL_CLOCK,0,mv1);
	}
	restore_flags(flags);
	return config;
}
/*
 * ---------------------------------------------------------------------------
 *
 * mv1_ics_out --
 * 	
 * 	writing to the registers of the ICS-1522 PLL-Chip
 *
 * ---------------------------------------------------------------------------
 */
static  void
mv1_ics_out(u16 data,u16 reg,MV1000 *mv1){
        u16 value = (reg&7) +((data&2047)<<3);
        int i;
        u_long flags;
        save_flags(flags);
        cli();
	mv1_pboutw(SERIAL_CLOCK,0,mv1);
        mv1_pboutw(0,CK1522_CS,mv1);
	mv1_pboutw(SERIAL_CLOCK,CK1522_CS,mv1);
        for(i=0;i<14;i++) {
                mv1_pboutw(value&1,CK1522_CS,mv1);
                mv1_pboutw((value&1)|SERIAL_CLOCK,CK1522_CS,mv1);
                value >>= 1;
        }
        mv1_pboutw(0,0,mv1);
	mv1_pboutw(SERIAL_CLOCK,0,mv1); // writes on SERIAL_CLOCK positive edge !!
        restore_flags(flags);
}
/*
 * ----------------------------------------------------------------------------
 *
 * mv1_32f8102_out --
 * 
 * 	writing to the registers of the Silicon Systems 32f8102	filter chip
 *
 * ----------------------------------------------------------------------------
 */
static void
mv1_32f8102_out(u8 data, u8 reg, MV1000 *mv1){
	int i;
	u_long flags;
	save_flags(flags);
	cli();
	mv1_pboutw(SERIAL_CLOCK,0,mv1);
	mv1_pboutw(0,FILTER_CS,mv1);
	mv1_pboutw(SERIAL_CLOCK,FILTER_CS,mv1);
	for(i=0;i<7;i++){
		mv1_pboutw(reg&1,FILTER_CS,mv1);
                mv1_pboutw((reg&1)|SERIAL_CLOCK,FILTER_CS,mv1);
                reg >>= 1;
	}
	mv1_pboutw(1|SERIAL_CLOCK,FILTER_CS,mv1);
	for(i=0;i<8;i++){
		mv1_pboutw(data&1,FILTER_CS,mv1);
                mv1_pboutw((data&1)|SERIAL_CLOCK,FILTER_CS,mv1);
                data >>= 1;
	}
	mv1_pboutw(SERIAL_CLOCK,0,mv1);
	restore_flags(flags);
}
/*
 * --------------------------------------------------------------------------------
 *	interrupt handling
 * --------------------------------------------------------------------------------
 */

static inline void 
mv1_set_frame_end_irq(int flag,MV1000 *bs){
	if(flag) 
		bs->misc_ctl |= VID_INT_ENA_MASK;
	else 
		bs->misc_ctl &= ~VID_INT_ENA_MASK;
	mv1_extoutl(bs->misc_ctl,MISC_CONTROL,bs);	
}
static inline int
mv1_inquire_frame_end_irq(MV1000 *bs){
	return (bs->misc_ctl & VID_INT_ENA_MASK)>>25;
}

static inline int
mv1_is_frame_end_irq(MV1000 *bs) {
        return mv1_inl(MISC_CONTROL,bs) & VID_INT_PND_R_MASK;
}

static inline void
mv1_clear_frame_end_irq(MV1000 *bs) {
        u_int value = bs->video_ctl | VID_INT_PND_MASK;
        mv1_extoutl(value,VIDEO_CONTROL,bs);
}

static inline void 
mv1_set_transfer_end_irq(int flag,MV1000 *bs){
        if(flag) 
                bs->misc_ctl |= MSTR_INT_ENA_MASK;
        else 
                bs->misc_ctl &= ~MSTR_INT_ENA_MASK;
        mv1_extoutl(bs->misc_ctl,MISC_CONTROL,bs);
}
static inline int
mv1_inquire_transfer_end_irq(MV1000 *bs){
        return (bs->misc_ctl & MSTR_INT_ENA_MASK)>>24;
}
static inline u_long
is_frame_end_irq(u_long pnd) {
	return pnd & VID_INT_PND_R_MASK;
}
static inline u_long
is_transfer_end_irq(u_long pnd) {
	return pnd & MSTR_INT_PND_R_MASK;
}
static inline u_long
is_bad_list_irq(u_long pnd) {
	return pnd & MSTR_BAD_LIST_R_MASK;
}
/*
 *	clear all irq requests and return the pending interrupts
 */
static  u_long 
mv1_clear_all_irqs(MV1000 *bs) {
	u_long pnd = mv1_inl(MISC_CONTROL,bs) & ALL_INTERRUPTS_MASK_R;
	u_long clr=0;
	if(is_frame_end_irq(pnd))  	clr |= VID_INT_PND_MASK;
	if(is_transfer_end_irq(pnd))    clr |= MSTR_INT_PND_MASK;
	if(is_bad_list_irq(pnd))	clr |= MSTR_BAD_LIST_MASK; 
	mv1_extoutl(clr|bs->video_ctl,VIDEO_CONTROL,bs);
	return pnd;
}
static inline void
mv1_force_clear_irqs(MV1000 *bs){
	mv1_extoutl(ALL_INTERRUPTS_MASK_W | bs->video_ctl,VIDEO_CONTROL,bs);
}
static inline void // ?????????????????????
mv1_select_sdc(u_long sdc,MV1000 *bs) {
	u_int value = 
		(bs->misc_ctl  & ~(1<<17))| ((sdc&1)<<17);
	mv1_extoutl(value,MISC_CONTROL,bs);
	bs->misc_ctl = value;
}
static  void
mv1_set_master_enable(int state,MV1000 *mv1) {
	u_int val = (mv1->h_v_start & ~MASTER_GO_MASK) |((state <<31)&MASTER_GO_MASK);
	if(state && !mv1->tlist_valid) {
		printk("mv1000: warning tried to enable master with invalid transferlist\n");
		mv1->h_v_start &= ~MASTER_GO_MASK;
	}
	mv1_extoutl(val,H_V_START,mv1);
	mv1->h_v_start = val;	
}
static int
mv1_inquire_master_enable(MV1000 *bs) {
	return (bs->h_v_start & MASTER_GO_MASK)>>31;
}
static  void
mv1_set_master_loop(int state,MV1000 *bs) {
	u_int val = (bs->h_v_start & ~MASTER_LOOP_MASK) |((state <<MASTER_LOOP_OFFSET)&MASTER_LOOP_MASK);
	mv1_extoutl(val,H_V_START,bs);
	bs->h_v_start = val;	
}
static  inline int
mv1_inquire_master_loop(MV1000 *bs) {
	return (bs->h_v_start & MASTER_LOOP_MASK)>>MASTER_LOOP_OFFSET;
}
static void
mv1_set_master_wait_frame(int wait,MV1000 *bs) {
	u_int val = (bs->h_v_start & ~MSTR_WAIT_FRAME_MASK) |
		((wait <<MSTR_WAIT_FRAME_OFFSET)&MSTR_WAIT_FRAME_MASK);
	mv1_extoutl(val,H_V_START,bs);
	bs->h_v_start = val;	
	
}
static  inline int
mv1_inquire_master_wait_frame(MV1000 *bs) {
	return (bs->h_v_start & MSTR_WAIT_FRAME_MASK)>>MSTR_WAIT_FRAME_OFFSET;
}
static inline int
mv1_is_irq_pending(MV1000 *bs){
	return mv1_inl(MISC_CONTROL,bs) 
		& ( VID_INT_PND_R_MASK | MSTR_INT_PND_R_MASK | MSTR_BAD_LIST_MASK);
}
/*
 * --------------------------------------------------------------------------
 *
 * --------------------------------------------------------------------------
 */
static inline int 
mv1_is_10_bit(MV1000 *bs){
	return !(bs->config_reg & ADC_TYPE);
}

static void
mv1_recalc_curr_gw(MV1000 *bs);
static int 
mv1_set_h_start(u_long hstart,MV1000 *bs){
	u_long value = (bs->h_v_start & ~(1023<<16) ) ;
	value |= ((hstart & ~3) << (16-2));
	mv1_extoutl(value,H_V_START,bs); 
	bs->h_v_start = value;
	bs->h_start = hstart;
	mv1_recalc_curr_gw(bs);
	return 0;
}
static int 
mv1_inquire_h_start(MV1000 *mv1){
	return mv1->h_start;
}
static int 
mv1_set_h_end(u_long hend,MV1000 *bs){
	u_long value = (bs->h_v_end & ~(1023<<16) ) ;
	value |= (hend & ~3) << (16-2);
	mv1_extoutl(value,H_V_END,bs);
	bs->h_v_end = value;
	bs->h_end = hend;
	mv1_recalc_curr_gw(bs);
	return 0;
}
static int 
mv1_inquire_h_end(MV1000 *bs){
	return bs->h_end;
}
static int
mv1_set_v_start(u_long vstart,MV1000 *bs){
	u_long value = bs->h_v_start & ~(4095 << 2);
	value |= (vstart&4095)<<2; 
        mv1_extoutl(value,H_V_START,bs);
	bs->h_v_start = value;
        bs->v_start = vstart;
	mv1_recalc_curr_gw(bs);
        return 0;
}
static int
mv1_inquire_v_start(MV1000 *bs){
        return bs->v_start;
}
static int
mv1_set_v_end(u_long vend,MV1000 *bs){
	u_long value = bs->h_v_end & ~(4095 << 2);
	value |= (vend & 4095)<<2;
        mv1_extoutl(value,H_V_END,bs);
	bs->h_v_end = value;
        bs->v_end = vend;
	mv1_recalc_curr_gw(bs);
        return 0;
}
static int
mv1_inquire_v_end(MV1000 *bs){
        return bs->v_end;
}

/*
 * -----------------------------------------------------------
 *
 * inquire_cur_line
 *      returns the current line. CURR_LINE is counted
 *      upwards even during the blank period.
 *
 * ----------------------------------------------------------
 */
static int
mv1_inquire_cur_line(MV1000 *bs){
        return (mv1_inl(CURR_LINE,bs)>>2) & ((1<<14)-1);
}
static int
mv1_set_eq_remove(u_long eq,MV1000 *bs){
	u_long u = bs->misc_ctl & ~(63<<26);
	u |= eq << 26;
        mv1_extoutl(u,MISC_CONTROL,bs);
	bs->misc_ctl	= u;
        bs->eq_remove = eq; 
        return 0;
}
static int
mv1_inquire_eq_remove(MV1000 *bs){
        return bs->eq_remove;
}


/*
 * 
 *
 * --------------------------------------------------------------------------
 */
static void
mv1_select_pixel_mode(u_long pm,MV1000 *bs) {
	u_char invmodeh[] = { 0,0x9a,0xf,0x1f,0xf };
	u_char invmodel[] = { 0,0   ,0  ,0   ,1 };
	u_short value = bs->lut_ctl;
	if(pm>4) return;
	value &= ~( (1<<4) | (0x9f<<8) );
	value |= ( (invmodel[pm]<<4) | (invmodeh[pm]<<8) );
	mv1_lutctl_out(value, bs);
	bs->lut_ctl = value; 
}
static int
mv1_inquire_pixel_mode(MV1000 *bs) {
	u_short value = bs->lut_ctl;
	static u_char mode[] = {0,0,2,4,1,0,3,0};
	value = ((value>>10)&4) | ((value >> 9 ) & 2) | ((value>>4) &1); 
	return mode[value];
}
static int
mv1_inquire_pixel_bits(MV1000 *bs) {
	static u_char pbits[] = {8,10,8,16,32}; // ??????????
	return pbits[mv1_inquire_pixel_mode(bs)];
}
static void
mv1_select_transfer_mode(u_long tmode,MV1000 *bs) {
	return ;
}
static void 
mv1_set_cvideo_source(u_long source,MV1000 *bs) {
	u_long value = 
	(bs->vctl & ~(0xf << 8)) | (( (1<<source)&0xf) << 8);
	mv1_vctl_outw(value,bs);
}
static int
mv1_inquire_cvideo_source(MV1000 *bs) {
	u_long reg = bs->vctl;
	return ffz( ~( (reg>>8) & 0xf ) );
}
static void 
mv1_set_csync_source(ulong source,MV1000 *bs){
	u_short value =
	(bs->vctl & ~(0xf << 12)) | (( (1<<source)&0xf) << 12);
	mv1_vctl_outw(value,bs); 
}
static int
mv1_inquire_csync_source(MV1000 *bs) {
	 u_long reg = bs->vctl;
         return ffz( ~((reg>>12) & 0xf ) ) ;
}
static void
mv1_select_clock_source(u_long source,MV1000 *bs) {
	u_short value = (bs->vctl & (~7) )|(source & 7) ;
	mv1_vctl_outw(value,bs); 
}
static int
mv1_inquire_clock_source(MV1000 *bs) {
	u_long reg = bs->vctl & 7;
	return reg;
}
static void      
mv1_select_sync_strip_range(u_long range,MV1000 *bs) {
	static u_char tab[] = { 0,8,12,14 };
 	u_long u = tab[range&3];
	u_long value = 
	(bs->vctl & ~(0xf << 3)) | ((u&0xf) << 3);
	mv1_vctl_outw(value,bs);
}
static int
mv1_inquire_sync_strip_range(MV1000 *bs) {
	static u_char invtab[] = { 0,0,0,0,0,0,0,0,1,1,1,1,2,2,3,0,0};
	u_long value = (bs->vctl >>3 ) & 0xf; 
	return invtab[value]; 
}

/*
static void
mv1_select_ad_convertor(u_long conv,MV1000 *bs){
	u_long value = (bs->video_ctl & ~(1<<7))| ((conv&1)<<7);
	mv1_outl(value,VIDEO_CONTROL,bs);
	bs->video_ctl = value;
}
*/

// Nothing will be done if not 10Bit AD
static void
mv1_set_shift_down(u_long flag,MV1000 *bs){
	if(!mv1_is_10_bit(bs))
		return;
	bs->vctl &= ~SHIFT_DOWN_2_MASK;
	if(flag)
		bs->vctl |= SHIFT_DOWN_2_MASK;
	mv1_vctl_outw(bs->vctl,bs);
}
static int
mv1_inquire_shift_down(MV1000 *bs){
	return (bs->vctl & SHIFT_DOWN_2_MASK)>>7;
}
static void
mv1_set_pll_phase_trim(u_long value,MV1000 *bs) {
	mv1_dac_out(value&255,DAC0,bs) ;
}
static int
mv1_inquire_pll_phase_trim(MV1000 *bs) {
	return bs->dac[0]; 
}

/* 
 * ---------------------------------------
 *  mv1_rd_vsync_info
 * 	returns 0 if in VSYNC
 *      doesn't work for external SYNC
 * --------------------------------------
 */
static int
mv1_rd_vsync_info(MV1000 *bs) {
	return (mv1_inl(MISC_CONTROL,bs) & VSYNC_STATE_MASK)>>VSYNC_STATE_OFFSET;
}
/*
 * -----------------------------------------------------
 *
 * mv1_is_offscreen
 * returns TRUE if the grabber is not in the image area
 *
 * ----------------------------------------------------
 */
static int
mv1_is_offscreen(MV1000 *bs) {
	int line = mv1_inquire_cur_line(bs);
	return((line < bs->v_start) || (line > bs->v_end));
}
static void
mv1_set_ad_top_level(ulong level,MV1000 *bs) {
	mv1_dac_out(level&0xff,DAC1,bs); 
}
static int
mv1_inquire_ad_top_level(MV1000 *bs) {
        return bs->dac[1];
}
static void
mv1_set_ad_bottom_level(ulong level,MV1000 *bs) {
	mv1_dac_out(level&0xff,DAC2,bs); 
}
static int
mv1_inquire_ad_bottom_level(MV1000 *bs) {
        return bs->dac[2];
}

static int 
mv1_wait_frame_end(MV1000 *bs)
{
	interruptible_sleep_on(&bs->frame_end_wait);
	if (signal_pending(current)) 
		return -EINTR;
	return 0;
} 

static int 
mv1_wait_transfer_end(MV1000 *bs)
{
	interruptible_sleep_on(&bs->transfer_end_wait);
	if (signal_pending(current)) 
		return -EINTR;
	return 0;
}

static void
mv1_set_clamping_level(ulong level,MV1000 *bs) {
        mv1_dac_out(level&255,DAC3,bs) ;
}

static int
mv1_inquire_clamping_level(MV1000 *bs) {
        return bs->dac[3];
}
static void
mv1_select_clamping(u_long source,MV1000 *bs) {
	u_char c[] = {0,2,1,3}; 
        u_long value = (bs->video_ctl & ~(3<<26))| ((c[source&3])<<26);
        mv1_extoutl(value,VIDEO_CONTROL,bs);
        bs->video_ctl = value;

}
static int
mv1_inquire_clamping(MV1000 *bs) {
	u_char c[] = {0,2,1,3};
	return c[(bs->video_ctl>>26) & 3];
}
static void
mv1_set_dc_offset(ulong level,MV1000 *bs) {
        mv1_dac_out(level&255,DAC4,bs) ;
}
static int
mv1_inquire_dc_offset(MV1000 *bs) {
        return bs->dac[4];
}
static int
mv1_inquire_field(MV1000 *bs) {
	return (mv1_inl(MISC_CONTROL,bs) >> 18) & 1;
}
static int
mv1_is_frame_end(MV1000 *bs) {
	return (mv1_inl(MISC_CONTROL,bs) & ONE_GRABBED_MASK); // ???????????
}

#define mv1_set_pll_phase_detector ics_set_vco_gain
/*
static void 
mv1_set_pll_phase_detector(u_long val,MV1000 *bs){

} */
static void
mv1_set_ls_exp_reduction(){

}
static int
mv1_set_ext_trigger(u_long mode,MV1000 *bs){
	u_long value = (bs->misc_ctl & ~(1<<19))| ((mode&1)<<19);
        mv1_extoutl(value,MISC_CONTROL,bs);
        bs->misc_ctl = value;
	return 0;
}
static inline int
mv1_inquire_ext_trigger(MV1000 *bs){
	return (bs->misc_ctl>>19) & 1;
}
static int
mv1_set_trigger_polarity(u_long pol,MV1000 *bs) {
        u_long value = (bs->misc_ctl & ~(1<<22))| ((pol&1)<<22);
        mv1_extoutl(value,MISC_CONTROL,bs);
        bs->misc_ctl = value;
	return 0;
}
static inline int
mv1_inquire_trigger_polarity(MV1000 *bs){
	return (bs->misc_ctl>>22) &1;
}

static inline void 
mv1_set_interlace(u_long flag,MV1000 *bs) {
	bs->video_ctl= (bs->video_ctl & ~INTERLACE_MASK)|((flag&1)<<19);
	mv1_extoutl(bs->video_ctl,VIDEO_CONTROL,bs);
	bs->curr_gw.interlaced = (flag &1);
	mv1_recalc_curr_gw(bs);
}
static inline int
mv1_inquire_interlace(MV1000 *bs) {
	return (bs->video_ctl & INTERLACE_MASK)>>19; 
}
/* doesn't work. When is EXT_TRIGGER_IN set ??? */
static int
mv1_wait_ext_trigger(u_long poll,MV1000 *bs) { 
	if(mv1_inquire_ext_trigger(bs) == 0)
		return 0; // Not external triggered 
	while((mv1_inl(MISC_CONTROL,bs) & (TRIGGER_WAIT_MASK)) !=0 ){
		if(signal_pending(current)) 
			return -EINTR;
		if(poll | (mv1_inquire_frame_end_irq(bs)==0))
			schedule();
		else
			interruptible_sleep_on(&bs->frame_end_wait);
	}
	return 1;
}
static void
mv1_wait_grab_done() {

}
static void
mv1_set_mem_mask(u_long mask,MV1000 *bs){
	u_long flags;
	save_flags(flags);	
	cli();	
	mv1_extoutl(mask,LUT_ADDR_MASK,bs);
	bs->mem_mask = mask;
	restore_flags(flags);
}
static inline unsigned long 
mv1_inquire_mem_mask(MV1000 *bs) {
	return  bs->mem_mask;
}
static void
mv1_set_grab_mask(u_long val,MV1000 *bs) {
	u_long value = (bs->h_v_start & ~(VIDEO_MASK_MASK))| 
		((val<<VIDEO_MASK_OFFSET)&VIDEO_MASK_MASK);
        mv1_extoutl(value,H_V_START,bs);
        bs->h_v_start = value;
}
static inline int
mv1_inquire_grab_mask(MV1000 *bs) {
        return  (bs->h_v_start&VIDEO_MASK_MASK)>>VIDEO_MASK_OFFSET;
}
static void
mv1_set_host_mask(u_long val,MV1000 *bs) {
        u_long value = (bs->h_v_start & ~1) | (val&1);
        mv1_extoutl(value,H_V_START,bs);
        bs->h_v_start = value;
}
static inline int
mv1_inquire_host_mask(MV1000 *bs) {
        return (bs->h_v_start >>0) & 1;
}
static inline void
mv1_set_grab_start_addr(u_long adr,MV1000 *bs){
	mv1_extoutl(adr ,GRAB_START_ADDR,bs);
	bs->grab_start_addr = adr;
}
static void
mv1_set_filter_pdc(u_long flag,MV1000 *bs){
	u_long value =  (flag &1);
        mv1_32f8102_out(value,SSI_PDC,bs);
        bs->filter_pdc = value;
}
static void
mv1_set_filter_cutoff(u_long cutoff,MV1000 *bs) {
	u_long value =  (cutoff &127);
	mv1_32f8102_out(value,SSI_DACF,bs);
	bs->freq_cutoff = value;
}
static inline int
mv1_inquire_filter_cutoff(MV1000 *bs){
	return bs->freq_cutoff;
}
static void
mv1_set_high_freq_boost(u_long boost,MV1000 *bs) {
	u_long value =(boost &127);
	mv1_32f8102_out(value,SSI_DACB,bs);
	bs->boost = value;
}
static inline int
mv1_inquire_high_freq_boost(MV1000 *bs){
	return bs->boost;
}
static inline void
mv1_set_gp_bits(int val,MV1000 *bs) {
	bs->video_ctl &= ~(GENERAL_0_MASK | GENERAL_1_MASK);
	bs->video_ctl |= (GENERAL_0_MASK | GENERAL_1_MASK)*(val&1);
	mv1_extoutl(bs->video_ctl,VIDEO_CONTROL,bs);
}
static void
mv1_set_gp_bit0(int val,MV1000 *bs){
	bs->video_ctl &= ~GENERAL_0_MASK;
	if(!val)	
		bs->video_ctl |= GENERAL_0_MASK;
	mv1_extoutl(bs->video_ctl,VIDEO_CONTROL,bs);
}

static inline int 
mv1_inquire_gp_bit0(MV1000 *bs){
        return 1-((bs->video_ctl & GENERAL_0_MASK) >> GENERAL_0_OFFSET);
}

static void
mv1_set_gp_bit1(int val,MV1000 *bs){
        bs->video_ctl &= ~GENERAL_1_MASK;
        if(!val)	
		bs->video_ctl |= GENERAL_1_MASK;
        mv1_extoutl(bs->video_ctl,VIDEO_CONTROL,bs);
}

static inline int
mv1_inquire_gp_bit1(MV1000 *bs){
        return 1-((bs->video_ctl & GENERAL_1_MASK) >> GENERAL_1_OFFSET);
}

static void
mv1_set_lut_mode(u_long mode,MV1000 *bs) {
		
}
static inline void
mv1_select_lut_bank(u_long lut,MV1000 *bs) {
	u_short value = bs->lut_ctl & ~7;
	value |= lut&7;
	mv1_lutctl_out(value,bs);
	bs->lut_ctl = value;
}
static inline int 
mv1_inquire_lut_bank(MV1000 *bs){
	return bs->lut_ctl &7;
}
/*
 * ------------------------------------------------------------------------
 *
 * mv1_write_lut_value
 *
 *	writes a value into a lookup table 
 *	the number of the current lut is saved, and restored  
 *
 *      Load the current lut with 1:1 or inverse mapping
 *      In 8 Bit Mode address is :
 *      addr =   addr_lane0 +       ( addr_lane1 << 8 ) 
 *             +(addr_lane2 <<16) + ( addr_lane3 << 24)
 *
 *      value is:
 *      val =     val_lane0 +          ( val_lane1 << 8 )
 *            + ( val_lane2 << 16) +   ( val_lane3 << 24)
 *       
 * -----------------------------------------------------------
 */

static void
mv1_write_lut_value(u_long value,u_long addr,int lut,u_long timeout,MV1000 *bs) {
	u_long flags;
	int current_lut = mv1_inquire_lut_bank(bs);
	int done = 0;
	save_flags(flags);
	while(!done) {
		cli();
		if(mv1_is_offscreen(bs)||(jiffies > timeout)) {
			if(lut != current_lut) mv1_select_lut_bank(lut,bs);
			mv1_extoutl(addr,LUT_ADDR_MASK,bs);
			mv1_extoutl(value,LUT_DATA,bs);
			mv1_extoutl(bs->mem_mask,LUT_ADDR_MASK,bs);
			done++;
		}
		restore_flags(flags);
	}
	if(lut!=current_lut) mv1_select_lut_bank(current_lut,bs);
}
/*
 * -----------------------------------------------------------
 *
 * mv1_load_lut_default --
 *
 * -----------------------------------------------------------
 */

static int 
mv1_load_lut_default(u_long flag,MV1000 *bs) {
	u_int i,j;
	int lut = mv1_inquire_lut_bank(bs);
	for(i=0;i<256;i++){
		if(flag) j = 255 -i;
		else j=i;
		j = j+(j<<8)+(j<<16)+(j<<24);
		mv1_write_lut_value(i+(i<<8)+(i<<16)+(i<<24),j,lut,jiffies,bs);
	}
	return 0;
}
static int   
mv1_set_grabflag(int gflag,MV1000 *bs){
	bs->video_ctl &= ~SINGLE_FRAME_MASK ;
        if(gflag == MV1_SINGLE_GRAB)
                bs->video_ctl |= SINGLE_FRAME_MASK;
	mv1_extoutl(bs->video_ctl,VIDEO_CONTROL,bs);
	return 0;
}
static int
mv1_inquire_grabflag(MV1000 *bs) {
	if(bs->video_ctl & SINGLE_FRAME_MASK)
		return MV1_SINGLE_GRAB;
	return MV1_CONT_GRAB;
}
static int
mv1_set_startfield(u_long ifield,MV1000 *bs) {
	bs->video_ctl &= ~( EVEN_START_MASK | ODD_START_MASK);
	if(ifield == MV1_GRAB_EVEN) {
                bs->video_ctl |= EVEN_START_MASK;
		bs->curr_gw.start_even = 1;
	}
        else  {
                bs->video_ctl |= ODD_START_MASK;
		bs->curr_gw.start_even = 0;
	}
        if(ifield == MV1_GRAB_NEXT)
                bs->video_ctl |= ODD_START_MASK | EVEN_START_MASK;
	mv1_extoutl(bs->video_ctl,VIDEO_CONTROL,bs);
	mv1_recalc_curr_gw(bs);
	return 0;
}
static int
mv1_inquire_startfield(MV1000 *bs){
	u_long u = bs->video_ctl & (EVEN_START_MASK | ODD_START_MASK);
	if(u==(EVEN_START_MASK | ODD_START_MASK))
		return MV1_GRAB_NEXT;
	else if (u==EVEN_START_MASK)
		return MV1_GRAB_EVEN;
	else if (u==ODD_START_MASK)
		return MV1_GRAB_ODD;
	return -1; //  What to do in this case ?
}
static int 
mv1_set_grab(u_long arg,MV1000 *bs) {
	bs->video_ctl &= ~CAPTURE_ENABLE_MASK;
	if(arg)
		bs->video_ctl |= CAPTURE_ENABLE_MASK;
	mv1_extoutl(bs->video_ctl,VIDEO_CONTROL,bs);
	return 0;
}
static int
mv1_inquire_grab(MV1000 *bs){
	return (bs->video_ctl & CAPTURE_ENABLE_MASK)>>15;
}


/*
 * ------------------------------------------------------------------------
 *
 * PLL-Control
 *
 * -----------------------------------------------------------------------
 */
 
static void
ics_set_fbkdiv(u_short div,MV1000 *bs) {
	u_short value; 
	value = (div-1)&2047;
	if(value >= 63)
		mv1_ics_out(value,0,bs);
	bs->icsreg[0] = value;
}
static int 
ics_inquire_fbkdiv(MV1000 *bs){
	return  bs->icsreg[0]+1;
}
static void
ics_set_refdiv(u_short div,MV1000 *bs) {
	u_short value =  bs->icsreg[3] & ~1023;
	value |= ((div-1) & 1023);
        mv1_ics_out(value,3,bs);
        bs->icsreg[3] = value;
}
static int
ics_inquire_refdiv(MV1000 *bs){
        return  (bs->icsreg[3] & 1023)+1;
}
static void
ics_set_refpol(u_long pol,MV1000 *bs){
	u_short value = (bs->icsreg[3] & ~(1<<10))|( (pol&1) << 10);
	mv1_ics_out(value,3,bs);
        bs->icsreg[3] = value;
}
static int
ics_inquire_refpol(MV1000 *bs){
	return (bs->icsreg[3]>>10) & 1;
}
static void
ics_set_vco_gain(u_long gain,MV1000 *bs){
	u_short v = (bs->icsreg[4] & ~7)|(gain&7);
	mv1_ics_out(v,4,bs);
	bs->icsreg[4] = v;
}
static int
ics_inquire_vco_gain(MV1000 *bs){
	return bs->icsreg[4] & 7;
}
static void
ics_set_pfd_gain(u_long gain,MV1000 *bs){
	u_short v = (bs->icsreg[4] & ~(7<<3)) | ((gain&7)<<3);
	mv1_ics_out(v,4,bs);
	bs->icsreg[4] = v;
}
static int
ics_inquire_pfd_gain(MV1000 *bs){
	return (bs->icsreg[4]>>3)&7;
}
static void 
ics_set_pden(u_long flag,MV1000 *bs) {
	u_short v = (bs->icsreg[4] & ~(1<<6)) | ((flag&1)<<6);
        mv1_ics_out(v,4,bs);
        bs->icsreg[4] = v;

}
static int
ics_inquire_pden(MV1000 *bs){
        return (bs->icsreg[4]>>6)&1;
}
static void
ics_set_clksel(u_long flag,MV1000 *bs) {
        u_short v = (bs->icsreg[4] & ~(1<<9)) | ((flag&1)<<9);
        mv1_ics_out(v,4,bs);
        bs->icsreg[4] = v;

}
static int
ics_inquire_clksel(MV1000 *bs){
        return (bs->icsreg[4]>>9)&1;
}

//	Output post-scaler
static void
ics_set_pda(u_long scale,MV1000 *bs) {
        u_short v = (bs->icsreg[5] & ~(3<<4)) | ((scale&3)<<4);
        mv1_ics_out(v,5,bs);
        bs->icsreg[5] = v;
}
static int
ics_inquire_pda(MV1000 *bs){
        return (bs->icsreg[5]>>4)&3;
}

// 	Fedback post scaler
static void
ics_set_pdb(u_long scale,MV1000 *bs) {
        u_short v = (bs->icsreg[5] & ~(3<<6)) | ((scale&3)<<6);
        mv1_ics_out(v,5,bs);
        bs->icsreg[5] = v;

}
static int
ics_inquire_pdb(MV1000 *bs){
        return (bs->icsreg[5]>>6)&3;
}
static inline void 
ics_set_lead_lag(u_long ldlg,MV1000 *bs) { 
	u_short v = (bs->icsreg[5] & ~(1 << 8)) | ((ldlg&1)<<8);
	mv1_ics_out(v,5,bs);
        bs->icsreg[5] = v;
}
static inline int
ics_inquire_lead_lag(MV1000 *bs) {
	return (bs->icsreg[5] >> 8)&1;
}
/*
 * -----------------------------------------------------------
 *
 * lm1882 Sync generator
 *
 * -----------------------------------------------------------
 */

static void
lm1882_select_sync_output(int mode,MV1000 *bs) {
	u_short v;
	u_short nv;
	if(mode == LM1882_CSYNC ) {
		v = 0;
	}
	else if(mode == LM1882_HVSYNC) {
		v = 2;
	}
	else return;
	nv = (bs->lm1882[0] & ~7) | v;
	lm1882_out(nv,0,bs);
}
static int
lm1882_inquire_sync_output(MV1000 *bs) {
	u_short v = bs->lm1882[0] & 7;
	if(v==0){
		return LM1882_CSYNC;	
	}
	else if(v==2) {
		return LM1882_HVSYNC;
	}
	else return -1;
}
static void
lm1882_select_eqserr(int mode,MV1000 *bs){
	u_short v=0;
	u_short mask = (3<<3) + (1<<9);
	
	if(mode == LM1882_NO_EQSERR) {
		v=1<<9;	
	}
	else if(mode == LM1882_DOUBLE_INTERLACED_EQSERR) {
		v=0<<3;
	}
	else if(mode == LM1882_DOUBLE_NI_EQSERR){
		v=1<<3;
	}
	else if(mode == LM1882_SINGLE_NI_EQSERR){
		v=3<<3;	
	}
	lm1882_out((bs->lm1882[0] & ~mask) | v,0,bs);

}
static int
lm1882_inquire_eqserr(MV1000 *bs) {
	u_short v = bs->lm1882[0] & ((3<<3)+(1<<9));
	if(v==(1<<9)) {
		return LM1882_NO_EQSERR;
	}
	else if(v == 0) {
		return LM1882_DOUBLE_INTERLACED_EQSERR;
	}
	else if(v==(1<<3)) {
		return LM1882_DOUBLE_NI_EQSERR;
	}
	else if(v==(3<<3)) {
		return LM1882_SINGLE_NI_EQSERR;
	}
	else return -1;
}
static void  
lm1882_select_vcsync_pol(int pol,MV1000 *bs){
	pol=(pol&1)<<6;
	lm1882_out( (bs->lm1882[0] & ~(1<<6)) | pol,0,bs);
}
static int
lm1882_inquire_vcsync_pol(MV1000 *bs) {
	return  (bs->lm1882[0]>>6) & 1;
}
static void  
lm1882_select_hsync_pol(int pol,MV1000 *bs){
	pol=(pol&1)<<8;
	lm1882_out( (bs->lm1882[0] & ~(1<<8)) | pol,0,bs);
}
static int
lm1882_inquire_hsync_pol(MV1000 *bs) {
	return  (bs->lm1882[0]>>8) & 1;
}

static void  
lm1882_set_pixel_per_line(int pixel,MV1000 *bs){
	lm1882_out( pixel,4,bs);
}
static int
lm1882_inquire_pixel_per_line(MV1000 *bs) {
	return  bs->lm1882[4] & 0xfff;
}
static void  
lm1882_set_line_per_frame(int line,MV1000 *bs){
	lm1882_out( line,8,bs);
}
static int
lm1882_inquire_line_per_frame(MV1000 *bs) {
	return  bs->lm1882[8] & 0xfff;
}
/*
 * -------------------------------------------------------------------------
 *
 *	MV1100 Digital Camera Module Functions
 *	currently disabled, because I don't know the adress of the MV1000
 *	control register
 * 
 * -------------------------------------------------------------------------
 */
static void
mv11_set_pulse_mode(int mode,MV1000 *bs){
	u_long nv = (bs->ext_vctl & ~MV1100_PULSE_MODE_MASK)|((mode&1)<<MV1100_PULSE_MODE_OFFSET); 
	mv11_out(nv,bs);
}
static int
mv11_inquire_pulse_mode(MV1000 *bs){
	return (bs->ext_vctl & MV1100_PULSE_MODE_MASK) >> MV1100_PULSE_MODE_OFFSET; 
}

static void
mv11_set_mode_control(int mc,MV1000 *bs){
	mc = ((mc&1)<<13) + ((mc&6)<<18);
	mv11_out((bs->ext_vctl & ~MV1100_MODE_CONTROL_MASK) | mc,bs);
}
static int
mv11_inquire_mode_control(MV1000 *bs){
	u_long mc = (bs->ext_vctl & MV1100_MODE_CONTROL_MASK);
	return ((mc>>13) & 1) + ((mc >> 18) & 6);
}
static void
mv11_select_par_frame_rem(int par,MV1000 *bs){
	u_long nv = (bs->ext_vctl & ~MV1100_PAR_FRAME_REM_MASK)|((par&1)<<MV1100_PAR_FRAME_REM_OFFSET);
	mv11_out(nv,bs);
}
static int
mv11_inquire_par_frame_rem(MV1000 *bs){
	return (bs->ext_vctl & MV1100_PAR_FRAME_REM_MASK)>>MV1100_PAR_FRAME_REM_OFFSET;

}

static void
mv11_select_clock(int clock,MV1000 *bs){
	u_long nv = (bs->ext_vctl & ~MV1100_CLOCK_SELECT_MASK)|((clock&3)<<MV1100_CLOCK_SELECT_OFFSET);
	mv11_out(nv,bs);
}
static int
mv11_inquire_clock(MV1000 *bs){
	return (bs->ext_vctl & MV1100_CLOCK_SELECT_MASK)>>MV1100_CLOCK_SELECT_OFFSET;
}
static void
mv11_set_bypass_counter(int byp,MV1000 *bs){
	u_long nv = (bs->ext_vctl & ~MV1100_BYPASS_COUNTER_MASK)|((byp&1)<<MV1100_BYPASS_COUNTER_OFFSET);
	mv11_out(nv,bs);
}
static int
mv11_inquire_bypass_counter(MV1000 *bs){
	return (bs->ext_vctl & MV1100_BYPASS_COUNTER_MASK)>>MV1100_BYPASS_COUNTER_OFFSET;
}
static void
mv11_select_camera_drive(int drive,MV1000 *bs){
	u_long nv = (bs->ext_vctl & ~MV1100_CAMERA_DRIVE_MASK)|((drive&1)<<MV1100_CAMERA_DRIVE_OFFSET);
	mv11_out(nv,bs);
}
static int
mv11_inquire_camera_drive(MV1000 *bs){
	return (bs->ext_vctl & MV1100_CAMERA_DRIVE_MASK)>>MV1100_CAMERA_DRIVE_OFFSET;
}

static void
mv11_set_counter(u_char counter,MV1000 *bs){
	mv11_out((bs->ext_vctl & ~MV1100_COUNTER_MASK)|counter,bs);
}
static int
mv11_inquire_counter(MV1000 *bs){
	return bs->ext_vctl & MV1100_COUNTER_MASK;
}

static void
mv11_set_trig_enable(int trig,MV1000 *bs){
	u_long nv = (bs->ext_vctl & ~MV1100_EXT_TRIG_MASK)|((trig&1)<<MV1100_EXT_TRIG_OFFSET);
	mv11_out(nv,bs);
}
static int
mv11_inquire_trig_enable(MV1000 *bs){
	return (bs->ext_vctl & MV1100_EXT_TRIG_MASK)>>MV1100_EXT_TRIG_OFFSET;
}

static void
mv11_set_trig_pol(int pol,MV1000 *bs){
	u_long nv = (bs->ext_vctl & ~MV1100_EXT_TRIG_POL_MASK)|((pol&1)<<MV1100_EXT_TRIG_POL_OFFSET);
	mv11_out(nv,bs);

}
static int
mv11_inquire_trig_pol(MV1000 *bs){
	return (bs->ext_vctl & MV1100_EXT_TRIG_POL_MASK)>>MV1100_EXT_TRIG_POL_OFFSET;
}

static void
mv11_set_ls_exp(int ls_exp,MV1000 *bs){
	u_long nv = (bs->ext_vctl & ~MV1100_MC1orINTEG_MASK)|((ls_exp&1)<<MV1100_MC1orINTEG_OFFSET);
	mv11_out(nv,bs);
}
static int
mv11_inquire_ls_exp(MV1000 *bs){
	return (bs->ext_vctl & MV1100_MC1orINTEG_MASK)>>MV1100_MC1orINTEG_OFFSET;

}
static void
mv11_set_ldv_pol(int pol,MV1000 *bs) {
	mv11_out((bs->ext_vctl & ~MV1100_LDV_POL_MASK) | ((pol&1)<<MV1100_LDV_POL_OFFSET),bs);	
}
static int
mv11_inquire_ldv_pol(MV1000 *bs) {
	return (bs->ext_vctl & MV1100_LDV_POL_MASK)>>MV1100_LDV_POL_OFFSET;	
}
static void
mv11_set_fdv_pol(int pol,MV1000 *bs) {
	mv11_out((bs->ext_vctl & ~MV1100_FDV_POL_MASK) | ((pol&1)<<MV1100_FDV_POL_OFFSET),bs);	
}
static int
mv11_inquire_fdv_pol(MV1000 *bs) {
	return (bs->ext_vctl & MV1100_FDV_POL_MASK)>>MV1100_FDV_POL_OFFSET;	
}
static void
mv11_set_pclk_pol(int pol,MV1000 *bs) {
	mv11_out((bs->ext_vctl & ~MV1100_PCLK_POL_MASK) | ((pol&1)<<MV1100_PCLK_POL_OFFSET),bs);	
}
static int
mv11_inquire_pclk_pol(MV1000 *bs) {
	return (bs->ext_vctl & MV1100_PCLK_POL_MASK) >> MV1100_PCLK_POL_OFFSET;	
}
static void 
mv1_set_top_sc(int ctl,MV1000 *bs) {
	bs->misc_ctl = (bs->misc_ctl & ~TOP_SC_EN_MASK) | ((ctl&1)<<TOP_SC_EN_OFFSET);
	mv1_extoutl(bs->misc_ctl,MISC_CONTROL,bs);
}
static int 
mv1_inquire_top_sc(MV1000 *bs) {
	return (bs->misc_ctl & TOP_SC_EN_MASK)>>TOP_SC_EN_OFFSET;
}

static int
test_ioctl1(u_long value,MV1000 *bs){
	mv1_ics_out(value,1,bs);
	return 0;
}
static int
test2_ioctl(u_long value,MV1000 *bs){
	bs->lut_ctl = value; 
	mv1_lutctl_out(value,bs);
	return 0;
}
static int
test2_get_ioctl(MV1000 *bs){
        return bs->lut_ctl;
}

/*
 * -------------------------------------------------------------------------
 *
 *	My own extensions
 *
 * -------------------------------------------------------------------------
 */

/*
 * ------------------------------------------------------------------------
 *
 * mv1_get_private_lut --
 *
 * 	Assigns a private lookup table to the caller.
 *	Don't forget to set pd->raw.lut on ioctl !
 *
 * ------------------------------------------------------------------------
 */
/*
static int
mv1_get_private_lut(MV1000 *bs) {
	int lut = ffz(bs->luts_private);
	if(lut >= NUM_LUTS) return -1; 
 	set_bit(lut,&(bs->luts_private));
	return lut;
}
static inline void
mv1_free_lut(int lut,MV1000 *bs) {
	clear_bit(lut,&(bs->luts_private));
}
*/
static void
mv1_set_integration_time(u_long value,MV1000 *bs){
	bs->i_time = value;
	if(bs->rest_time > value){
		bs->rest_time = 1;
	}  
}
static void
mv1_reset_integration_time(MV1000 *bs){
	bs->rest_time = -1;
}
static int
mv1_inquire_integration_time(MV1000 *bs){
	return bs->i_time;
}
static int
fg_inquire_rest_time(MV1000 *bs) {
	return bs->rest_time;
}
static int
cam_inquire_int_ctl_mode(MV1000 *bs) {
	return integration_mode;
}
static void
cam_set_int_ctl_mode(u_long mode,MV1000 *bs) {
	integration_mode = mode;
}
